@@ -42,9 +42,12 @@ namespace GNU_gama { namespace local {
 
       friend class AcordAlgorithm;
       friend class AcordAzimuth;
-      friend class AcordHdiffs;
+      friend class AcordZderived;
+      friend class AcordHdiff;
+      friend class AcordVector;
       friend class AcordPolar;
       friend class AcordTraverse;
+      friend class AcordIntersection;
       friend class AcordWeakChecks;
 
       struct Point
@@ -67,19 +70,25 @@ namespace GNU_gama { namespace local {
       std::pair<double, bool> get_dir(Observation* o);
       bool in_missingXY(PointID pt);
 
-      double MAX_NORM = 0.1;
+      // tolerance for rejecting median of xy / z
+      double median_max_norm_ = 0.1;
+
       bool slope_observations_ = false;
+      bool has_azimuths_ = false;
 
       PointData&       PD_;
       ObservationData& OD_;
       std::vector<StandPoint*> SPClusters_;
+      std::vector<HeightDifferences*> HDiffClusters_;
+      std::vector<Vectors*> VectorsClusters_;
       std::set<PointID> missing_xy_;
       std::set<PointID> missing_z_;
-      std::multimap<PointID,LocalPoint> same_points_xy_;
-      std::multimap<PointID,LocalPoint> same_points_z_;
+      std::multimap<PointID,LocalPoint> candidate_xy_;
+      std::multimap<PointID,double>     candidate_z_;
       std::multimap<PointID,Observation*> obs_from_;
       std::multimap<PointID,Observation*> obs_to_;
       bool get_medians();
+      void get_medians_z();
       size_type new_points_xy_, new_points_z_;
 
       using Traverse = std::vector<Point>;
@@ -95,7 +104,7 @@ namespace GNU_gama { namespace local {
       std::vector<std::shared_ptr<AcordAlgorithm>> algorithms_;
 
 #ifdef DEBUG_ACORD2
-      void debug_info(const char* text) const;
+      void debug_info(std::string text) const;
 #endif
     };
 
