@@ -31,7 +31,7 @@
 
 #include <gnu_gama/local/results/text/underline.h>
 #include <gnu_gama/local/network.h>
-#include <gnu_gama/local/acord.h>
+#include <gnu_gama/local/acord/acord.h>
 #include <gnu_gama/gon2deg.h>
 #include <gnu_gama/utf8.h>
 #include <cctype>
